#!/usr/bin/env python
#
# wrapper for MEEP
# Author: Jing Ouyang, Purdue University, 2007
#
import sys
import os
import re
import shutil
import stat

import Rappture
from Rappture.tools import getCommandOutput as RapptureExec

def getstdout(): # get the standard output of a mpi job
    cmd = 'find . -name "*.stdout" -print'
    fid = os.popen(cmd).readlines()
    if len(fid) >= 1:
        return 'Stdout is:\n'+open(fid[0][:-1]).read()
    else:
        return 'Error: no returned stdout file\n'


def getstderr(): # get the standard error of a mpi job
    cmd = 'find . -name "*.stderr" -print'
    fid = os.popen(cmd).readlines()
    if len(fid) >= 1:
        return 'Stderr is:\n'+open(fid[0][:-1]).read()
    else:
        return ''

exitStatus = 0
io=Rappture.library(sys.argv[1])

band = io.get('input.string(deck).current')
displayFluxes = re.search(".*display-fluxes (.*)\)",band)
if displayFluxes:
   fluxTitles = displayFluxes.group(1).split()

# create the directory where output should be placed
re_driverNum  = re.compile(r'[0-9]+')
driverNum = re_driverNum.search(os.path.split(sys.argv[1])[1]).group()
resultDirName = os.path.join(os.getcwd(),driverNum)
if os.path.exists(resultDirName):
   shutil.rmtree(resultDirName,True)
os.mkdir(resultDirName)

wd = os.getcwd()
os.chdir(resultDirName)

f=open('run.ctl','w')
f.write(band)
f.close()

ncpu = int(io.get('input.group(options).integer(ncpu).current'))
walltime = io.get('input.group(options).number(walltime).current')
walltime = int(float(Rappture.Units.convert(walltime,to='min',units='off')))

if ncpu > 1:
   if   ncpu > 48:
      venue = 'rcac-ncn-par'
   elif walltime > 120:
      venue = 'rcac-ncn-par'
   elif ncpu*walltime > 3840:
# 32*120
      venue = 'rcac-ncn-par'
   else:
      venue = 'rcac-ncn-hub'

   submitScriptName = 'submit_meep.sh'
   submitScript     = """#!/bin/sh

trap cleanup HUP INT QUIT ABRT TERM

cleanup()
{
   echo "Abnormal termination by signal"
   kill -s TERM `jobs -p`
}

"""

   submitScript += "submit -v %s -n %s -w %s \\\n" % (venue,ncpu,walltime)
   submitScript += "       meep-1.2-mpi run.ctl &\n"
   submitScript += "wait %1\n"
   submitScript += "exit $?\n"

   submitScriptPath = os.path.join(resultDirName,submitScriptName)
   f = open(submitScriptPath, 'w')
   f.write(submitScript)
   f.close()
   os.chmod(submitScriptPath,stat.S_IRWXU|stat.S_IRGRP|stat.S_IXGRP|stat.S_IROTH|stat.S_IXOTH)

   exitStatus,submitOut,runerr = RapptureExec([submitScriptPath],True)
   runout = getstdout()
   runerr += getstderr()
else:
   exitStatus,runout,runerr = RapptureExec(["meep1","run.ctl"],True)

io.put('output.string(runout).current',runout)
io.put('output.string(runout).about.label','Output')
fpOut = open('run.stdout','w')
fpOut.write(runout)
fpOut.close()

if runerr != '':
    io.put('output.string(runerr).current',runerr)
    io.put('output.string(runerr).about.label','Error')
    fpErr = open('run.stderr','w')
    fpErr.write(runerr)
    fpErr.close()

if not exitStatus:
    fluxData = ""
    for line in runout.splitlines(True):
        if re.match("^flux[0-9]*:,",line):
            fluxData += line
    if fluxData != "":
        fpFlux = open('fluxdata.csv','w')
        fpFlux.write(fluxData)
        fpFlux.close()

#       io.put('output.string(fluxdata).current',fluxData)
#       io.put('output.string(fluxdata).about.label','Flux Data')

        line = fluxData.splitlines()[0]
        nFields = line.count(',')
        curveData = {}
        for field in xrange(nFields):
            curveData[field] = ""
        for line in fluxData.splitlines():
            fields = line.split(',')
            for field in xrange(nFields):
                curveData[field] += "%s %s\n" % (fields[1],fields[1+field])
        for field in xrange(1,nFields):
            fluxPath = 'output.curve(flux%d)' % (field)
            io.put(fluxPath + '.about.label',fluxTitles[field-1])
            io.put(fluxPath + '.xaxis.label','frequency')
            io.put(fluxPath + '.yaxis.label',fluxTitles[field-1])
            io.put(fluxPath + '.component.xy',curveData[field])

    cmd = 'find . -name "*.h5" -print'
    h5files = os.popen(cmd).readlines()
    h5files.sort()
    for h5file in h5files:
       h5lsStatus,h5lsOut,h5lsErr = RapptureExec(["h5ls","%s" % (h5file[:-1])])
       if h5lsStatus == 0:
          is2D = True
          h5lsOut = h5lsOut.strip()
          for dataset in h5lsOut.split('\n'):
             dimensions = re.search(".*{(.*)}.*",dataset)
             if dimensions:
                if dimensions.group(1).count(',') != 1:
                   is2D = False
             else:
                is2D = False

          if is2D:
             h5pngStatus,h5pngOut,h5pngErr = RapptureExec(["h5topng","-c","dkbluered","%s" % (h5file[:-1])])
             if h5pngStatus == 0:
                validImage = False
                h5png = "%s.png" % (h5file[2:-4])
                pngStatus,pngOut,pngErr = RapptureExec(["file","%s" % (h5png)])
                if pngStatus == 0:
                   dimensions = re.search(".* ([0-9]+) x ([0-9]+),.*",pngOut)
                   if dimensions:
                      if int(dimensions.group(1)) > 1 and int(dimensions.group(2)) > 1:
                         # remove leading ./ and trailing .png^M
                         io.put('output.image(%s).about.label' % (h5png[:-4]),h5png[:-4])
                         io.put('output.image(%s).current' % (h5png[:-4]),h5png,type='file', compress=True)
                         validImage = True

                if not validImage:
                   if os.path.exists("%s.png" % (h5file[:-4])):
                      os.remove("%s.png" % (h5file[:-4]))

cmd = 'tar cfz meepresults.tar.gz ./run.ctl'
allFiles = os.listdir(resultDirName)
for fileType in 'stdout','stderr','h5','png','csv':
   reFiles = re.compile('.*.%s$' % (fileType))
   matchingFiles = filter(reFiles.search,allFiles)
   if len(matchingFiles) > 0:
      cmd += ' ./*.' + fileType

tarout = os.popen(cmd).readlines()
io.put('output.string(tarball).about.label','meepresults.tar.gz')
io.put('output.string(tarball).current','meepresults.tar.gz',type='file',compress=True)

os.chdir(wd)
shutil.rmtree(resultDirName,True)

# show submission errors to user as stdout/err log.
exitStatus = 0
io.result(exitStatus)
sys.exit(exitStatus)

